window.onload = () => {
	//
	let overview = document.getElementById("overview");
	let summary_list = overview.querySelector(".summary-list");
	let breakdown_list = overview.querySelector(".breakdown-list");
	let breakdown_current = document.getElementById("breakdown-current");
	const refresh_interval = setInterval(
		refresh_elements,
		500,
		overview,
		summary_list,
		breakdown_list,
		breakdown_current
	);
};

async function refresh_elements(
	overview,
	summary_list,
	breakdown_list,
	breakdown_current
) {
	fetch("http://localhost:8000/spoiler")
		.then((response) => response.json())
		.then(
			(data) => {
				// Attempt to receive response JSON.
				const response_object = JSON.parse(JSON.stringify(data));

				// Parse out data from the back-end into variables.
				const overview_checks_undiscovered =
					response_object.Totals.Checks.Undiscovered;
				const overview_checks_total = response_object.Totals.Checks.Total;
				const overview_entrances_undiscovered =
					response_object.Totals.Entrances.Undiscovered;
				const overview_entrances_total = response_object.Totals.Entrances.Total;
				const current_scene_name = response_object.Current.Scene;
				const all_scenes = response_object.Scenes;
				let new_breakdown_list = overview
					.querySelector(".breakdown-list")
					.cloneNode(true);
				let new_summary_list = overview
					.querySelector(".summary-list")
					.cloneNode(true);
				new_breakdown_list.innerHTML = "";
				new_summary_list.firstElementChild.innerHTML = "";

				// Refresh elements with new data from the back end.

				//  Clone all the needed elements for updating.
				let overview_checks_title = overview.querySelector(".overview-checks");
				let overview_entrances_title = overview.querySelector(
					".overview-entrances"
				);
				let summary_block =
					summary_list.firstElementChild.firstElementChild.cloneNode(true);
				let breakdown_block = breakdown_list.firstElementChild.cloneNode(true);
				breakdown_block.classList.remove("hidden");

				// Set content to updated data.
				overview_checks_title.textContent = `Checks: ${overview_checks_undiscovered}/${overview_checks_total}`;
				overview_entrances_title.textContent = `Entrances: ${overview_entrances_undiscovered}/${overview_entrances_total}`;

				// Create new lists with updated data.
				Object.keys(all_scenes).forEach((scene) => {
					// Create variables for element pointers.
					summary_block =
						summary_list.firstElementChild.firstElementChild.cloneNode(true);
					summary_block.classList.remove("hidden");
					let summary_title = summary_block.querySelector(".summary-title");
					let summary_checks = summary_block.querySelector(".summary-checks");
					let summary_entrances =
						summary_block.querySelector(".summary-entrances");

					breakdown_block = breakdown_list.firstElementChild.cloneNode(true);
					breakdown_block.classList.remove("hidden");
					let breakdown_block_title = breakdown_block.querySelector(
						".breakdown-block-title"
					);
					let breakdown_block_checks_title = breakdown_block.querySelector(
						".breakdown-block-checks-title"
					);
					let new_breakdown_block_checks_list = breakdown_block
						.querySelector(".breakdown-block-checks-list")
						.cloneNode(true);
					let breakdown_block_checks_list_item = breakdown_block
						.querySelector(".breakdown-block-checks-list")
						.firstElementChild.cloneNode(true);
					breakdown_block_checks_list_item.classList.remove("hidden");
					let breakdown_block_entrances_title = breakdown_block.querySelector(
						".breakdown-block-entrances-title"
					);
					let new_breakdown_block_entrances_list = breakdown_block
						.querySelector(".breakdown-block-entrances-list")
						.cloneNode(true);
					let breakdown_block_entrances_list_item = breakdown_block
						.querySelector(".breakdown-block-entrances-list")
						.firstElementChild.cloneNode(true);
					breakdown_block_entrances_list_item.classList.remove("hidden");
					let new_breakdown_block_mapped_list = breakdown_block
						.querySelector(".breakdown-block-mapped-list")
						.cloneNode(true);
					let breakdown_block_mapped_list_item = breakdown_block
						.querySelector(".breakdown-block-mapped-list")
						.firstElementChild.cloneNode(true);
					breakdown_block_mapped_list_item.classList.remove("hidden");

					// Create variables for commonly used values.
					let scene_checks_undiscovered =
						all_scenes[scene].Totals.Checks.Undiscovered;
					let scene_checks_total = all_scenes[scene].Totals.Checks.Total;
					let scene_entrances_undiscovered =
						all_scenes[scene].Totals.Entrances.Undiscovered;
					let scene_entrances_total = all_scenes[scene].Totals.Entrances.Total;

					// Set textContent.
					summary_title.textContent = scene;
					summary_checks.textContent = `Checks: ${scene_checks_undiscovered}/${scene_checks_total}`;
					summary_entrances.textContent = `Entrances: ${scene_entrances_undiscovered}/${scene_entrances_total}`;
					breakdown_block_title.textContent = scene;
					breakdown_block_checks_title.textContent = `Checks: ${all_scenes[scene].Totals.Checks.Undiscovered}/${all_scenes[scene].Totals.Checks.Total}`;
					breakdown_block_entrances_title.textContent = `Entrances: ${all_scenes[scene].Totals.Entrances.Undiscovered}/${all_scenes[scene].Totals.Entrances.Total}`;

					// Create checks, entrances, and mapped entrances lists.
					Object.keys(all_scenes[scene].Checks).forEach((check) => {
						if (!all_scenes[scene].Checks[check]) {
							breakdown_block_checks_list_item.textContent = `❌ ${check}`;
							new_breakdown_block_checks_list.appendChild(
								breakdown_block_checks_list_item.cloneNode(true)
							);
						}
					});
					Object.keys(all_scenes[scene].Entrances).forEach((entrances) => {
						if (all_scenes[scene].Entrances[entrances] == "") {
							breakdown_block_entrances_list_item.textContent = `❌ ${entrances}`;
							new_breakdown_block_entrances_list.appendChild(
								breakdown_block_entrances_list_item.cloneNode(true)
							);
						} else {
							breakdown_block_mapped_list_item.textContent = `✔️ ${entrances} -> ${all_scenes[scene].Entrances[entrances]}`;
							new_breakdown_block_mapped_list.appendChild(
								breakdown_block_mapped_list_item.cloneNode(true)
							);
						}
					});

					// Apply color coding to summary block
					summary_block.classList.remove(
						"bg-green-900",
						"bg-yellow-900",
						"bg-blue-900",
						"bg-red-900"
					);
					if (
						scene_checks_undiscovered > 0 &&
						scene_entrances_undiscovered > 0
					) {
						if (scene_entrances_undiscovered == scene_entrances_total) {
							summary_block.classList.add("bg-red-900");
						} else {
							summary_block.classList.add("bg-green-900");
						}
					} else if (scene_checks_undiscovered > 0) {
						summary_block.classList.add("bg-yellow-900");
					} else if (scene_entrances_undiscovered > 0) {
						summary_block.classList.add("bg-blue-900");
					}

					// Replace lists
					breakdown_block
						.querySelector(".breakdown-block-checks-list")
						.replaceWith(new_breakdown_block_checks_list);
					breakdown_block
						.querySelector(".breakdown-block-entrances-list")
						.replaceWith(new_breakdown_block_entrances_list);
					breakdown_block
						.querySelector(".breakdown-block-mapped-list")
						.replaceWith(new_breakdown_block_mapped_list);

					// Append relevant elements to lists.
					if (scene == current_scene_name) {
						summary_block.classList.add("hidden");
						breakdown_current.firstElementChild.replaceWith(
							breakdown_block.cloneNode(true)
						);
						breakdown_block.classList.add("hidden");
					} else if (
						scene_checks_undiscovered <= 0 &&
						scene_entrances_undiscovered <= 0
					) {
						summary_block.classList.add("hidden");
						breakdown_block.classList.add("hidden");
					}
					new_summary_list.firstElementChild.appendChild(
						summary_block.cloneNode(true)
					);
					new_breakdown_list.appendChild(breakdown_block.cloneNode(true));
				});

				// Replace with new data.
				summary_list.replaceWith(new_summary_list.cloneNode(true));
				breakdown_list.replaceWith(new_breakdown_list.cloneNode(true));
				document
					.getElementById("breakdown-current")
					.replaceWith(breakdown_current.cloneNode(true));
			},
			(error) => {
				return error;
			}
		);
}

// Outdated funcion to log the data gathered from the backend.
function log_elements() {
	console.log(overview.checks.textContent);
	console.log(overview.entrances.textContent);
	console.log(current_scene.name.textContent);
	console.log(current_scene.checks.title.textContent);
	for (i in current_scene.checks.list.children) {
		if (current_scene.checks.list.children[i].textContent) {
			console.log(current_scene.checks.list.children[i].textContent);
		}
	}
	console.log(current_scene.entrances.title.textContent);
	for (i in current_scene.entrances.list.children) {
		if (current_scene.entrances.list.children[i].textContent) {
			console.log(current_scene.entrances.list.children[i].textContent);
		}
	}
	for (i in current_scene.entrances.mapped.children) {
		if (current_scene.entrances.mapped.children[i].textContent) {
			console.log(current_scene.entrances.mapped.children[i].textContent);
		}
	}
}
